# decompyle3 version 3.9.0
# Python bytecode version base 3.7.0 (3394)
# Decompiled from: Python 3.7.16 (default, Jan 17 2023, 09:28:58)
# [Clang 14.0.6 ]
# Embedded file name: output/Live/mac_universal_64_static/Release/python-bundle/MIDI Remote Scripts/ableton/v3/control_surface/components/__init__.py
# Compiled at: 2023-11-21 10:21:18
# Size of source mod 2**32: 3998 bytes
from .accent import AccentComponent
from .active_parameter import ActiveParameterComponent
from .auto_arm import AutoArmComponent
from .background import BackgroundComponent, ModifierBackgroundComponent, TranslatingBackgroundComponent
from .channel_strip import ChannelStripComponent
from .clip_actions import ClipActionsComponent
from .clip_slot import ClipSlotComponent
from .clipboard import ClipboardComponent
from .device import DeviceComponent
from .device_bank_navigation import DeviceBankNavigationComponent
from .device_parameters import DeviceParametersComponent
from .drum_group import DEFAULT_DRUM_TRANSLATION_CHANNEL, DrumGroupComponent, DrumPadClipboardComponent
from .drum_group_scroll import DrumGroupScrollComponent
from .grid_resolution import GRID_RESOLUTIONS, GridResolutionComponent
from .loop_selector import LoopSelectorComponent
from .mixer import MixerComponent, SendIndexManager
from .note_editor import DEFAULT_STEP_TRANSLATION_CHANNEL, NoteEditorComponent, PitchProvider
from .page import Pageable, PageComponent
from .paginator import NoteEditorPaginator, Paginator
from .playable import PlayableComponent
from .playhead import PlayheadComponent
from .recording import (
    BasicRecordingMethod,
    NextSlotRecordingMethod,
    NextSlotWithOverdubRecordingMethod,
    RecordingComponent,
    RecordingMethod,
    ViewBasedRecordingComponent,
)
from .scene import SceneComponent
from .scroll import Scrollable, ScrollComponent
from .session import ClipSlotClipboardComponent, SessionComponent
from .session_navigation import SessionNavigationComponent
from .session_overview import SessionOverviewComponent
from .session_ring import SessionRingComponent
from .simple_device_navigation import SimpleDeviceNavigationComponent
from .sliced_simpler import DEFAULT_SIMPLER_TRANSLATION_CHANNEL, SlicedSimplerComponent
from .step_sequence import SequencerClip, StepSequenceComponent, create_sequencer_clip
from .target_track import ArmedTargetTrackComponent, TargetTrackComponent
from .transport import TransportComponent
from .undo_redo import UndoRedoComponent
from .view_control import ViewControlComponent
from .view_toggle import ViewToggleComponent

__all__ = (
    "DEFAULT_DRUM_TRANSLATION_CHANNEL",
    "DEFAULT_SIMPLER_TRANSLATION_CHANNEL",
    "DEFAULT_STEP_TRANSLATION_CHANNEL",
    "GRID_RESOLUTIONS",
    "AccentComponent",
    "ActiveParameterComponent",
    "ArmedTargetTrackComponent",
    "AutoArmComponent",
    "BackgroundComponent",
    "BasicRecordingMethod",
    "ChannelStripComponent",
    "ClipActionsComponent",
    "ClipboardComponent",
    "ClipSlotClipboardComponent",
    "ClipSlotComponent",
    "DeviceBankNavigationComponent",
    "DeviceComponent",
    "DeviceParametersComponent",
    "DrumGroupComponent",
    "DrumGroupScrollComponent",
    "DrumPadClipboardComponent",
    "GridResolutionComponent",
    "LoopSelectorComponent",
    "MixerComponent",
    "ModifierBackgroundComponent",
    "NextSlotRecordingMethod",
    "NextSlotWithOverdubRecordingMethod",
    "NoteEditorComponent",
    "NoteEditorPaginator",
    "Pageable",
    "PageComponent",
    "Paginator",
    "PitchProvider",
    "PlayableComponent",
    "PlayheadComponent",
    "RecordingComponent",
    "RecordingMethod",
    "SceneComponent",
    "Scrollable",
    "ScrollComponent",
    "SendIndexManager",
    "SequencerClip",
    "SessionComponent",
    "SessionNavigationComponent",
    "SessionOverviewComponent",
    "SessionRingComponent",
    "SimpleDeviceNavigationComponent",
    "SlicedSimplerComponent",
    "StepSequenceComponent",
    "TargetTrackComponent",
    "TranslatingBackgroundComponent",
    "TransportComponent",
    "UndoRedoComponent",
    "ViewBasedRecordingComponent",
    "ViewControlComponent",
    "ViewToggleComponent",
    "create_sequencer_clip",
)
