# decompyle3 version 3.9.0
# Python bytecode version base 3.7.0 (3394)
# Decompiled from: Python 3.7.16 (default, Jan 17 2023, 09:28:58)
# [Clang 14.0.6 ]
# Embedded file name: output/Live/mac_universal_64_static/Release/python-bundle/MIDI Remote Scripts/ableton/v2/control_surface/roar_decoration.py
# Compiled at: 2023-12-21 15:35:35
# Size of source mod 2**32: 1023 bytes
from .decoration import LiveObjectDecorator
from .internal_parameter import EnumWrappingParameter


class RoarDeviceDecorator(LiveObjectDecorator):
    def __init__(self, *a, **k):
        (super().__init__)(*a, **k)
        self._additional_parameters = self._create_parameters()
        self.register_disconnectables(self._additional_parameters)

    @property
    def parameters(self):
        return tuple(self._live_object.parameters) + self._additional_parameters

    def _create_parameters(self):
        return (
            EnumWrappingParameter(
                name="Routing",
                parent=self,
                values_host=(self._live_object),
                index_property_host=self,
                values_property="routing_mode_list",
                index_property="routing_mode_index",
            ),
        )
